# WCM SimpleSAMLphp Authentication

This module provides utilites, configuration templates, and setup scripting for SimpleSAMLphp authentication on Pantheon hosted Drupal 10 sites. It requires a Composer based Drupal installation and uses composer scaffold hooks to write configuration files to the correct location. It is pre-configured for the OSU SSO service and centralizes all required configuration to a single file stored in your private files. It also provides the following addtional functionality: 

- Cron metadata refresh. Each time Drupal cron runs, the metadata will be validated and refreshed if expired.
- Default role assignment. In addition to the attribute role mapping provided by drupal/simplesamlphp_auth, a default role (other than Authenticated User) can be assigned upon successful login. 
- Additional logging. Metadata status is reported to the Drupal logs. 

#### Upgrading from 3.0.x
Version 3.1 of this module adds the [simpleSAMLphp AuthCrypt](https://simplesamlphp.org/docs/contrib_modules/authcrypt/authcrypt.html) module to enable password hashing. This module must be enabled in private/simplesamlphp/config/config.php. Either delete this file and allow the updated version to be scaffolded, or update your existing config.php module.enable section. 
```
    'module.enable' => [
        'exampleauth' => false,
        'core' => true,
        'admin' => true,
        'saml' => true,
        'metarefresh' => true,
        'cron' => true,
        authcrypt' => true,
    ],
```

#### Upgrading from 2.x
The configuration templates have been updated for the 3.x version of this module. In order for them to be scaffolded into the correct location, you'll need to first delete the old templates from private/simplesamlphp/config. Files to delete: authsources.php, config.php, example-parameters.php, module_cron.php, and module_metarefresh.php. 

The location of the simplesamlphp web UI files has changed. Delete the web/simplesaml symlink so a new one can be created. 

A new `production` parameter has been added. It determines the SimpleSAMLphp operation mode and must be set in your parameters.php file. See the updated example-parameters.php file for the complete set of required config parameters. 

### Installation
1. Add the wcm-osu repository to your composer.json.
```
    "repositories": [
        {   
            "type":"composer",
            "url":"https://code.osu.edu/api/v4/group/5741/-/packages/composer/packages.json"
        }
    ],
```

2. Add code.osu.edu to the gitlab-domains setting in the config section of your composer.json.
```
    "config": {
        "gitlab-domains": [
            "code.osu.edu"
        ]
    }
```

3. Enable file scaffolding by adding this project to the allowed-packages section of the drupal-scaffold extras key in your composer.json file. You will also need to define the web-root location.
```
"extra": {
    "drupal-scaffold": {
        "allowed-packages": [
            "wcm-osu/wcm_simplesamlphp_auth"
        ],
        "locations": {
            "web-root": "./web"
        }
    }
},
```

4. Ensure that your root /private directory is protected in your pantheon.yml (or pantheon.uptream.yml). https://docs.pantheon.io/pantheon-yml#protected-web-paths
```
protected_web_paths:
  - /private/
```

5. Require with composer.
```
user@host> composer require wcm-osu/wcm_simplesamlphp_auth
```

6. Add the SimplesamlphpConfigurator class to the composer classmap to make it accessable to composer event hooks. 
```
"autoload": {
    "classmap": [
        "scripts/wcm-osu/composer/SimplesamlphpConfigurator.php"
    ]
},
```

7. Hook the post scaffold event by adding a post-drupal-scaffold-cmd key the scripts section of your composer.json and pointing it to the configureForPantheon method of the SimplesamlphpConfigurator.
```
"scripts": {
    "post-drupal-scaffold-cmd": [
        "wcm_site_scripts\\composer\\SimplesamlphpConfigurator::configureForPantheon"
    ]
}
```

8. Run `composer install`. New files will be created at scripts/wcm-osu/composer/ and private/simplesamlphp/config. Add the new files to your site repository. Commit and push your changes. 


### Configure SimpleSAMLphp

The configuration templates included with this module use the Pantheon private files location for configuration parameters, certificates, and metadata. 

1. In a location outside of your site repository, generate new certificate files. Verify that sp-cert.crt and sp-key.pem have been created.
```
user@host /user/home $ openssl req -newkey rsa:2048 -new -x509 -days 3652 -nodes -out sp-cert.crt -keyout sp-key.pem
```

2. Make a copy of `example-parameters.php` to a new file `parameters.php`. Set the Pantheon site name and primary domain if applicable. Set the entityID to https://HOSTNAME/simplesaml. The cronKey and secretSalt values need to be random 54 character strings which can be generated by a utility like http://textmechanic.com/Random-String-Generator.html. The adminPassword should be a hash generated by the included simpleSAMLphp utility: `vendor/simplesamlphp/simplesamlphp/bin/pwgen.php`. Do not use plain text. See https://simplesamlphp.org/docs/contrib_modules/authcrypt/authcrypt.html for more info on AuthCrypt. Enter values for the keys under the site specific and team/maintainer sections. The rest of the parameters are preconfigured for OSU Shibboleth SSO and Pantheon environments and should not need to be modified unless your site has a unique configuration.

3. Connect to your site via SFTP. Upload your certificates to /files/private/simplesamlphp/certs/ and your parameters.php to /files/private/simplesamlphp/config/. Create an empty metadata directory at /files/private/simplesamlphp/metadata/. Be aware that these files may need to be manaully copied between Pantheon environments. 

### Register Site for SSO
Your site must be registered with OSU before it will be able use the Shibboleth SSO service.

1. Navigate to https://[YOUR-HOSTNAME]/simplesaml/module.php/saml/sp/metadata.php/default-sp?output=xhtml

2. Copy the XML MetaData into a new text file. **You may need to go through and replace Pantheon dev domain names to match your final domain name.**

3. Submit a new Service-Now request using the "Web Log-In Service Provider Registration" form in the service catalog. Paste the contents of your XML Metadata file into the Self Signed Certificate field of the request form. 

### Fetch Metadata and Test Functionality
Once you've recieved confirmation that your site has been registered, generate metadata and test through the simpleSAMLphp web UI. 

1. Navigate to https://[YOUR-HOSTNAME]/simplesaml/module.php/admin and log in using the admin password that was previously configured. Click the Metarefresh link in the Details section of the Configuration page. You should see a log message confirming that the saml20-idp-remote.php file has been written.

2. Click the back button and then go to the Test tab. Click the default-sp link. This should redirect you the OSU Web log in page. Enter your OSU username and password. After successful authentication, you will be redirected back to your site's simpleSAMLphp web UI and shown the saml attributes returned from the SSO service. 

### Install Modules
At this point we have a functional simpleSAMLphp service provider running on our site. The last step is to enable and configure the Drupal modules that interact with the SP. 

1. Log in to your site as an administrator and install the WCM SimpleSAMLphp Auth module. This will also install several other required Drupal modules.

2. Navigate to Configuration->SimpleSAMLphp Auth Settings. Go to the User Info and Syncing tab. Enter the value `urn:oid:0.9.2342.19200300.100.1.3` for unique identifier, username, and email attributes. Alternatively, you can use any attribute string returned from the authentication test in the SimpleSAMLphp web UI to populate your user fields. 

3. Adjust other module settings as needed. When all of the settings have been entered, go to the Basic Settings and check the box for Activate Authentication via SimpleSAMLphp. Once enabled, https://[YOUR-HOSTNAME]/saml_login will redirect to the SSO service. 